/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.vew.bmvew.bmvew.meldung;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.bmvew.bmvew.BetriebsMeldungApp;

/**
 * Kommentar
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/05/21 14:56:51 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class BenutzerAktion
{
  /**
   * Der Debugger
   */
  Debug _debug;

  /**
   * Die Verbindung zum Datenverteiler
   */
  ClientDavInterface _connection;

  /**
   * Das Datenmodell
   */
  DataModel _datenmodell;

  /**
   * Konstruktor
   */
  public BenutzerAktion()
  {

    _debug = Debug.getLogger();
    _connection = BetriebsMeldungApp.getConnection();
    _datenmodell = _connection.getDataModel();
  }

  /**
   * Wertet die Benutzeraktion aus
   * 
   * @param result
   *          Der Datensatz der Benutzeraktion
   */
  public synchronized void auswerteBenutzerAktion(ResultData result)
  {

    // Daten aus der Benutzeraktion holen
    SystemObject so = result.getObject();

    Data data = result.getData();

    TextValue wichtigkeit = data.getItem("Wichtigkeit").asTextValue();
    TextValue status = data.getItem("Status").asTextValue();
    Long wiedervorlage = data.getItem("Wiedervorlage").asTimeValue().getMillis();

    Long time = result.getDataTime();

    TextValue text = data.getItem("BenutzerMeldungsAnteil").asTextValue();

    Data d = data.getItem("Urlasser");
    
    BenutzerAnteil anteil = new BenutzerAnteil(time, text, d);
    
    Array refs = data.getItem("MeldungReferenz").asArray();

    if (refs.getLength() == 0)
      throw new IllegalArgumentException("keine Referenz angegeben");

    for (int i = 0; i < refs.getLength(); i++)
    {
      ReferenceValue ref = refs.getItem(i).asReferenceValue();

      Meldung meldung = BetriebsMeldungApp.getMeldungList().get(ref.getSystemObject().getPid());

      // Infoanteil zur Liste der Meldung inkrementieren
            
      try
      {
        meldung.setWichtigkeit(wichtigkeit);
        meldung.setWiedervorlage(wiedervorlage);
        
        // Nur, wenn die Benuztermeldung einen zusätzlichen Meldungstext enthält, wird der
        // Benutzermeldungsanteil inkrementiert.
        if (text.getText().length() > 0)
          meldung.getBenutzerAnteilListe().add(anteil);
      }
      catch (RuntimeException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }

      if (!meldung.setStatus(status))
      {
        _debug.fine("Zustand konnte nicht verändert werden");
        continue;

      }
      else
        _debug.fine("Zustand wurde verändert");

//      if (status.getText().equals("erledigt"))
//        continue;

      AttributeGroup atg = _datenmodell.getAttributeGroup("atg.meldung");
      Aspect asp = _datenmodell.getAspect("asp.zustand");
      DataDescription dd = new DataDescription(atg, asp);

      // Arbeitskopie des Datums der atg erstellen
      data = meldung.getData().createModifiableCopy();

      data.getItem("Wichtigkeit").asTextValue().setText(meldung.getWichtigkeit().getText());

      data.getItem("Status").asTextValue().setText(meldung.getStatus().getText());

      data.getItem("Wiedervorlage").asTimeValue().setMillis(meldung.getWiedervorlage());

      Array array = data.getItem("BenutzerMeldungsAnteil").asArray();

      List<BenutzerAnteil> all = meldung.getBenutzerAnteilListe();

      array.setLength(all.size());

      for (int j = 0; j < all.size(); j++)
      {
        BenutzerAnteil tmp = (BenutzerAnteil)all.get((all.size() - 1) - j);
        Long t = tmp.getZeit();
        String s = tmp.getText().getText();

        Data urlasserData = tmp.getUrlasser();

        array.getItem(j).getTimeValue("Zeitstempel").setMillis(t);
        array.getItem(j).getTextValue("MeldungsText").setText(s);

        array.getItem(j).getItem("Urlasser").getItem("BenutzerReferenz").asReferenceValue().setText(
            urlasserData.getItem("BenutzerReferenz").asReferenceValue().getText());
        array.getItem(j).getItem("Urlasser").getItem("Ursache").asTextValue().setText(
            urlasserData.getItem("Ursache").asTextValue().getText());
        array.getItem(j).getItem("Urlasser").getItem("Veranlasser").asTextValue().setText(
            urlasserData.getItem("Veranlasser").asTextValue().getText());

      }

      data.getItem("Urlasser").getItem("BenutzerReferenz").asReferenceValue().setText(
          meldung.getUrlasser().getItem("BenutzerReferenz").asReferenceValue().getText());
      data.getItem("Urlasser").getItem("Ursache").asTextValue().setText(
          meldung.getUrlasser().getItem("Ursache").asTextValue().getText());
      data.getItem("Urlasser").getItem("Veranlasser").asTextValue().setText(
          meldung.getUrlasser().getItem("Veranlasser").asTextValue().getText());

      // Umkopieren der relevanten Betriebsmeldungsdaten in die Arbeitskopie ist beendet,
      // es kann nun der Datensatz der Meldung aktualisiert werden
      meldung.setData(data);

      // publiziere
      _debug.fine("publiziere Daten für Meldung: " + ref.getSystemObjectPid());
      Vector<ResultData> sendeDaten = BetriebsMeldungApp.getSendeDaten();
      ResultData res1 = new ResultData(ref.getSystemObject(), dd, System.currentTimeMillis(), data);

      synchronized (sendeDaten)
      {
        sendeDaten.add(res1);
        sendeDaten.notify();
      }

//      Sender sender = new Sender(_connection, SenderRole.source());
//      sender.sende(ref.getSystemObject(), atg, asp, data);

      // persistiere
      _debug.fine("persistiere Daten für Meldung: " + ref.getSystemObjectPid());
            
      
      atg = _datenmodell.getAttributeGroup("atg.meldung");
      asp = _datenmodell.getAspect("asp.parameterVorgabe");
      dd = new DataDescription(atg, asp);
      
      Vector<ResultData> paramDaten = BetriebsMeldungApp.getParamDaten();
      
      ResultData res2 = new ResultData(ref.getSystemObject(), dd, System.currentTimeMillis(), data);
      
      synchronized (paramDaten)
      {
        paramDaten.add(res2);        
        paramDaten.notify();
      }
    }

  }

}
