/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.vew.bmvew.bmvew.meldung;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.Data.TimeValue;

/**
 * Kapselt den Informationsanteil einer Meldung
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/09/24 12:50:45 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class InformationsAnteil
{
  /**
   * Der Zeitstempel
   */
  private Long _zeit;

  /**
   * Der Meldungstyp
   */
  private TextValue _typ;

  /**
   * Die Meldungsklasse
   */
  private TextValue _klasse;

  /**
   * Die Referenz
   */
  private ReferenceValue _referenz;

  /**
   * Der Status
   */
  private TextValue _status;

  /**
   * Der Meldungstext
   */
  private TextValue _text;

  /**
   * Der Urlasser
   */
  private Data _urlasser;

  /**
   * Konstruktor
   * 
   * @param zeit Zeit
   * @param typ Typ
   * @param klasse Klasse
   * @param referenz Referenz
   * @param status Status
   * @param text Text
   * @param urlasser Urlasser
   */
  public InformationsAnteil(Long zeit, TextValue typ, TextValue klasse, ReferenceValue referenz, TextValue status,
      TextValue text, Data urlasser)
  {
    if (zeit == null || typ == null || klasse == null || status == null || text == null || urlasser == null)
      throw new IllegalArgumentException();

    _zeit = zeit;
    _typ = typ;
    _klasse = klasse;
    
    if (referenz != null)
      _referenz = referenz;
    
    _status = status;
    _text = text;
    _urlasser = urlasser;

  }

  /**
   * Konstruktor
   * 
   * @param zeit Zeit
   * @param typ Typ
   * @param klasse Klasse
   * @param referenz Referenz
   * @param status Status
   * @param text Text
   * @param urlasser Urlasser
   */
  public InformationsAnteil(TimeValue zeit, TextValue typ, TextValue klasse, ReferenceValue referenz, TextValue status,
      TextValue text, Data urlasser)
  {
    if (zeit == null || typ == null || klasse == null || status == null || text == null || urlasser == null)
      throw new IllegalArgumentException();

    _zeit = zeit.getMillis();
    _typ = typ;
    _klasse = klasse;
    
    if (referenz != null)
      _referenz = referenz;
    
    _status = status;
    _text = text;
    _urlasser = urlasser;

  }

  /**
   * @param infoAnteil Data
   */
  public InformationsAnteil(Data infoAnteil)
  {
    if (infoAnteil == null)
      throw new IllegalArgumentException();

    _zeit = infoAnteil.getItem("Zeitstempel").asTimeValue().getMillis();
    _typ = infoAnteil.getItem("MeldungsTyp").asTextValue();
    _klasse = infoAnteil.getItem("MeldungsKlasse").asTextValue();

    Array refs = infoAnteil.getItem("Referenz").asArray();
    if (refs.getLength() > 0)
      _referenz = refs.getItem(0).asReferenceValue();

    _status = infoAnteil.getItem("MeldungsStatus").asTextValue();
    _text = infoAnteil.getItem("MeldungsText").asTextValue();
    _urlasser = infoAnteil.getItem("Urlasser");

    if (_zeit == null || _typ == null || _klasse == null || _status == null || _text == null || _urlasser == null)
      throw new IllegalArgumentException();

  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj)
  {
    // TODO Auto-generated method stub
    InformationsAnteil vergleich = (InformationsAnteil)obj;

    if (_zeit.equals(vergleich.getZeit()) && _typ.equals(vergleich.getTyp()) && _klasse.equals(vergleich.getKlasse())
        && _referenz.equals(vergleich) && _status.equals(vergleich.getStatus()) && _text.equals(vergleich.getText()))
    {

      return true;
    }

    return false;

  }

  public Long getZeit()
  {
    return _zeit;
  }

  public void setZeit(Long zeit)
  {
    _zeit = zeit;
  }

  public TextValue getTyp()
  {
    return _typ;
  }

  public void setMeldungsTyp(TextValue typ)
  {
    _typ = typ;
  }

  public TextValue getKlasse()
  {
    return _klasse;
  }

  public void setKlasse(TextValue klasse)
  {
    _klasse = klasse;
  }

  public ReferenceValue getReferenz()
  {
    return _referenz;
  }

  public void setReferenz(ReferenceValue referenz)
  {
    _referenz = referenz;
  }

  public TextValue getStatus()
  {
    return _status;
  }

  public void setStatus(TextValue status)
  {
    _status = status;
  }

  public TextValue getText()
  {
    return _text;
  }

  public void setText(TextValue text)
  {
    _text = text;
  }

  public Data getUrlasser()
  {
    return _urlasser;
  }

  public void setUrlasser(Data urlasser)
  {
    _urlasser = urlasser;
  }

}
